function GPS = ReadNMEA(filename)
%
% function to read GPS RMC data file
% Example: GPS_data = ReadNMEA('GPSLOG08.TXT');
%
% Output data structure:
% GPS_data = 
% 
%           Lat_num: [double] -- decimal latitude in degrees
%          Long_num: [double] -- decimal logitude in degrees
%     UTC_TimeStamp: {cell} -- UTC time string
%     EST_TimeStamp: {cell} -- EST time string
%             Speed: [double] -- ground speed in m/s
%           Heading: [double] -- heading in degrees
%
% Revision History:
%     9/27/09 HC (Version 1)
            
knot2ms = 0.5144444;    % conversion factor from knots to m/s

% read GPS data file
[ID, UTC, Status, Lat, NS, Long, EW, GS, Direction, Date, Mag, CS] = ...
    textread(filename,'%s%s%s%s%s%s%s%f%f%s%s%s','delimiter',',');

n = length(ID);

for i = 1:n
    % convert latitude data
    if strcmp(NS{i}, 'N')
        Lat_sign = 1;
    else
        Lat_sign = -1;
    end
    Lat_deg = str2num(Lat{i}(1:2));
    Lat_deg_frac = str2num(Lat{i}(3:end))/60;
    GPS.Lat_num(i) = Lat_sign * (Lat_deg + Lat_deg_frac);
    
    % conver longitude data
    if strcmp(EW{i}, 'E')
        Long_sign = 1;
    else
        Long_sign = -1;
    end
    Long_deg = str2num(Long{i}(1:3));
    Long_deg_frac = str2num(Long{i}(4:end))/60;
    GPS.Long_num(i) = Long_sign * (Long_deg + Long_deg_frac);
    
    % create timestamps
    dd = Date{i}(1:2);
    mm = Date{i}(3:4);
    yy = Date{i}(5:6);
    HH = UTC{i}(1:2);
    HH_EST = num2str(str2num(HH)-4);
    MM = UTC{i}(3:4);
    SS = UTC{i}(5:end);
    GPS.UTC_TimeStamp{i} = [mm,'/',dd,'/',yy,' ',HH,':',MM,':',SS];
    GPS.EST_TimeStamp{i} = [mm,'/',dd,'/',yy,' ',HH_EST,':',MM,':',SS];
    
    GPS.Speed(i) = GS(i) * knot2ms;
    GPS.Heading(i) = Direction(i);
end
